/*
   NNTP client
   Copyright (c) 1999  Martin Hinner <mhi@penguin.cz>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* This package contains some routines from popclient package, original
 * copyright follows:
 *
 * Copyright 1993,1994 by Carl Harris <ceharris@vt.edu>, Jr.
 * All rights reserved
 *
 * Distribute freely, except: don't remove my name from the source or
 * documentation (don't take credit for my work), mark your changes (don't
 * get me blamed for your possible bugs), don't alter or remove this
 * notice.  May be sold if buildable source is provided to buyer.  No
 * warrantee of any kind, express or implied, is included with this
 * software; use at your own risk, responsibility for damages (if any) to
 * anyone resulting from the use of this software rests entirely with the
 * user.
 */

#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>

#include "nntpclient.h"
#include "socket.h"

char *nntp_server = 0;
int nntp_port;
char *newsgroup;
char *mailbox;
char *newsrc;
int verbose;
int topipe;

char usage[] =
"Usage: %s [-v] [-p port] [-s nntp_server] [-r nntpclientrc_file]\n"
"            newsgroup [mailbox-file]\n"
"\n"
"\n"
"Default values:\n"
"    nntpclientrc_file     ~/.nntpclientrc\n"
"    nntp_server            $NNTPSERVER\n"
"    port                   119\n"
"    mailbox-file           ~/mail/{newsgroup}\n"
"\n"
"Report bugs to <mhi@penguin.cz>.\n";


int
main (int argc, char **argv)
{
  int c;

  nntp_port = 0;
  verbose = 0;
  topipe = 0;

  while ((c = getopt (argc, argv, "hp:s:r:Vv")) != -1)
    {
      switch (c)
	{
	case 'h':
	  fprintf (stderr, usage, argv[0]);
	  exit (0);
	case 'p':
	  nntp_port = atoi (optarg);
	  break;
	case 's':
	  nntp_server = optarg;
	  break;
	case 'r':
	  newsrc = optarg;
	  break;
	case 'v':
	  verbose = 1;
	  break;
	case 'V':
	  printf ("nntpclient version 0.1\n");
	  exit (0);
	default:
	  fprintf (stderr, usage, argv[0]);
	  exit (1);
	}
    }


  if (argc == optind)
    {
      fprintf (stderr, usage, argv[0]);
      exit (1);
    }

  if (argc > optind + 2)
    {
      fprintf (stderr, usage, argv[0]);
      exit (1);
    }

  newsgroup = argv[optind];
  if (argc > optind + 1)
    {
      mailbox = argv[optind + 1];
    }

  if (!nntp_server)
    nntp_server = getenv ("NNTPSERVER");

  if (!newsrc)
    {
      newsrc = (char *) malloc (1024);
      if (!newsrc)
	{
	  fprintf (stderr, "%s: Not enough memory\n", argv[0]);
	  exit (1);
	}
      strcpy (newsrc, getenv ("HOME"));
      strcat (newsrc, "/.nntpclientrc");
    }


  if (!mailbox)
    {
      mailbox = (char *) malloc (1024);
      if (!mailbox)
	{
	  fprintf (stderr, "%s: Not enough memory\n", argv[0]);
	  exit (1);
	}
      strcpy (mailbox, getenv ("HOME"));
      strcat (mailbox, "/mail/");
      strcat (mailbox, newsgroup);
    }

  if (nntp_port == 0)
    {
      nntp_port = 119;
    }

  if (mailbox[0] == '|')
    topipe = 1;

  if (!nntp_server)
    {
      fprintf (stderr, "%s: You must specify NNTP server\n", argv[0]);
      exit (1);
    }

  if (doNNTP (nntp_server, nntp_port, newsgroup, mailbox))
    return 1;

  return 0;
}

int
readmsg (int socket, int mboxfd, int topipe)
{
  char buf[1024];
  char *bufp;
  time_t now;

  /* Unix mail folder format requires the Unix-syntax 'From' header.
     POP3 doesn't send it, so we fake it here. */
  if (!topipe)
    {
      now = time (NULL);
      sprintf (buf, "From NNTPclient %s", ctime (&now));
      if (write (mboxfd, buf, strlen (buf)) < 0)
	{
	  perror ("readmsg: write");
	  return -1;
	}
    }

  /* read the message content from the server */
  while (1)
    {
      if (SockGets (socket, buf, sizeof (buf)) < 0)
	return -1;
      bufp = buf;
      if (*bufp == '.')
	{
	  bufp++;
	  if (*bufp == 0)
	    break;		/* end of message */
	}
      strcat (bufp, "\n");
      if (write (mboxfd, bufp, strlen (bufp)) < 0)
	{
	  perror ("readmsg: write");
	  return -1;
	}
    }

  if (!topipe)
    {
      /* The server may not write the extra newline required by the Unix
         mail folder format, so we write one here just in case */
      if (write (mboxfd, "\n", 1) < 0)
	{
	  perror ("readmsg: write");
	  return -1;
	}
    }
  else
    {
      /* The mail delivery agent may require a terminator. If it does, we have
         a problem... */
    }

  return (0);
}

int 
openpipe (char *cmd)
{
  int pipefd[2];
  int childpid;
  char binmailargs[80];
  char *argv[4];

  if (pipe (pipefd) < 0)
    {
      perror ("nntpclient: openpipe: pipe");
      return (-1);
    }
  if ((childpid = fork ()) < 0)
    {
      perror ("nntpclient: openpipe: fork");
      return (-1);
    }

  else if (childpid == 0)
    {
      /* in child process space */
      close (pipefd[1]);	/* close the 'write' end of the pipe */
      close (0);		/* get rid of inherited stdin */
      if (dup (pipefd[0]) != 0)
	{
	  fputs ("nntpclient: openpipe: dup() failed\n", stderr);
	  exit (1);
	}

      argv[0] = "sh";
      argv[1] = "-c";
      argv[2] = cmd;
      argv[3] = 0;

      execv ("/bin/sh", argv);

      /* if we got here, an error occurred */
      perror ("popclient: openmailpipe: exec");
      return -1;
    }

  /* in the parent process space */
  close (pipefd[0]);		/* close the 'read' end of the pipe */
  return pipefd[1];

}

int 
openfolder (char *filename)
{
  int fd;

  if (verbose)
   printf("Opening folder %s\n",filename);

  if (!strcmp (filename, "-"))
    return dup (STDOUT_FILENO);

  if (filename[0] == '|')
    return openpipe (filename + 1);

  if ((fd = open (filename, O_CREAT | O_WRONLY | O_APPEND, 0600)) >= 0)
    return (fd);

  perror ("nntpclient: openfolder: open");
  return (-1);
}
